#include "daq_config.h"

#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <string.h>

#include "nds_log.h"
#include "nds_logging.h"

#if _WIN32
static char* strsep(char** stringp, const char* delim);
#endif /* _WIN32 */

#define DEFAULT_VERBOSE_DEBUGGING 0

struct nds_log_ {
  int	s_group_mask;
  int	s_group_debug_level[ NDS_LOG_GROUP_SIZE_MAX ];
  nds_logging_function_type	s_logging_func;
  nds_flush_function_type	s_flush_func;
};

typedef struct nds_log_ nds_log_t;

static nds_log_t
log_info = {
  /* --------------------------------------------------------------------
   * Set bit mask of things that should be debugged by
   * default.
   * ----------------------------------------------------------------- */
#if DEFAULT_VERBOSE_DEBUGGING
  ( 1 << NDS_LOG_GROUP_CONNECTION )
  | ( 1 << NDS_LOG_GROUP_VERBOSE_ERRORS )
  | ( 1 << NDS_LOG_GROUP_TRACE_ENTRY_EXIT )
  | ( 1 << NDS_LOG_GROUP_STATUS_UPDATE )
#else	/* DEFAULT_VERBOSE_DEBUGGING */
  0
#endif /* DEFAULT_VERBOSE_DEBUGGING */
  ,
  /* --------------------------------------------------------------------
   * Setting of the default logging levels
   * ----------------------------------------------------------------- */
  {
    30,	/* NDS_LOG_GROUP_CONNECTION */
    30, /* NDS_LOG_GROUP_VERBOSE_ERRORS */
    20, /* NDS_LOG_GROUP_TRACE_ENTRY_EXIT */
    20, /* NDS_LOG_GROUP_STATUS_UPDATE */
    20, /* NDS_LOG_GROUP_USER */
  },
  (nds_logging_function_type)nds_log,
  (nds_flush_function_type)nds_flush
};


void
nds_function_flush( nds_flush_function_type Func )
{
  log_info.s_flush_func = Func;
}

void
nds_function_logging( nds_logging_function_type Func )
{
  log_info.s_logging_func = Func;
}

int
nds_logging_check( int Group, int Level )
{
  int retval = 0;

  if ( ( ( 1 << Group ) & log_info.s_group_mask )
       && ( log_info.s_group_debug_level[ Group ] >= Level ) )
  {
    retval = 1;
  }

  return retval;
}

void
nds_logging_enable( int Group )
{
  if ( ( Group < 0 )
       || ( Group >= NDS_LOG_GROUP_SIZE_MAX ) )
  {
    return;
  }
  log_info.s_group_mask |= ( 1<<Group );
}

void
nds_logging_disable( int Group )
{
  if ( ( Group < 0 )
       || ( Group >= NDS_LOG_GROUP_SIZE_MAX ) )
  {
    return;
  }
  log_info.s_group_mask &= ~( 1<<Group );
}

void
nds_logging_debug_level( int Group, int Level )
{
  if ( ( Group < 0 )
       || ( Group >= NDS_LOG_GROUP_SIZE_MAX ) )
  {
    return;
  }
  log_info.s_group_debug_level[ Group ] = Level;
}

void
nds_logging_flush( )
{
  if ( log_info.s_flush_func )
  {
    (*log_info.s_flush_func)( );
  }
}

void
nds_logging_init( )
{
  char *info = getenv("NDS_LOGGING");
  if ( info )
  {
    char *tok_name;
    char *tok_value;
    tok_name = strsep(&info, ":");
    while ( tok_name )
    {
      int nds_log_group = 0;
      int nds_log_level = -2;

      char *tmp;
      tmp = strsep(&tok_name, "=");
      if ( tmp )
      {
	tok_value = tok_name;
      }
      tok_name = tmp;
      /*
       * Determine what debugging information is being requested
       */
      if ( strcmp( tok_name, "CONNECTION" ) == 0 )
      {
	nds_log_group = NDS_LOG_GROUP_CONNECTION;
      }
      else if ( strcmp( tok_name, "VERBOSE_ERRORS" ) == 0 )
      {
	nds_log_group = NDS_LOG_GROUP_VERBOSE_ERRORS;
      }
      else if ( strcmp( tok_name, "TRACE_ENTRY_EXIT" ) == 0 )
      {
	nds_log_group = NDS_LOG_GROUP_TRACE_ENTRY_EXIT;
      }
      else if ( strcmp( tok_name, "STATUS_UPDATE" ) == 0 )
      {
	nds_log_group = NDS_LOG_GROUP_STATUS_UPDATE;
      }
      else if ( strcmp( tok_name, "USER" ) == 0 )
      {
	nds_log_group = NDS_LOG_GROUP_USER;
      }
      else
      {
	tok_name = strsep(&info, ":");
	continue;
      }
      /*
       * Determine quantity of output
       */
      if ( tok_value )
      {
	nds_log_level = atoi( tok_value );
	if ( nds_log_level < -2 )
	{
	  tok_name = strsep(&info, ":");
	  continue;
	}
      }
      switch ( nds_log_level )
      {
      case -2:
	nds_logging_enable( nds_log_group );
	break;
      case -1:
	nds_logging_disable( nds_log_group );
	break;
      default:
	nds_logging_debug_level( nds_log_group,
				 nds_log_level );
	nds_logging_enable( nds_log_group );
	break;
      }
      tok_name = strsep(&info, ":");
    }
  }
}

void
nds_logging_printf( const char* MessageFormat, ... )
{
  if ( log_info.s_logging_func )
  {
    char buf[2048];
    va_list	ap;

    va_start( ap, MessageFormat );

#if HAVE_VSNPRINTF_S
    vsnprintf_s
#else /* HAVE_VSNPRINTF_S */
    vsnprintf
#endif /* HAVE_VSNPRINTF_S */
      ( buf, sizeof( buf ),
#if HAVE_VSNPRINTF_S
	sizeof( buf ) - 1,
#endif /* HAVE_VSNPRINTF_S */
	MessageFormat, ap );

    (*log_info.s_logging_func)( buf );

    va_end( ap );
  }
}

#if _WIN32
static char*
strsep(char** stringp, const char* delim)
{
  char* start = *stringp;
  char* p;

  p = (start != NULL) ? strpbrk(start, delim) : NULL;

  if (p == NULL)
  {
    *stringp = NULL;
  }
  else
  {
    *p = '\0';
    *stringp = p + 1;
  }

  return start;
}
#endif /* _WIN32 */
